@@ -201,6 +201,11 @@ testException (test_TestFactory   factory,
 	objref = test_TestFactory_getBasicServer (factory, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
+	test_BasicServer_opException (CORBA_OBJECT_NIL, ev);
+	g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
+	g_assert (strcmp (CORBA_exception_id (ev), ex_CORBA_INV_OBJREF) == 0);
+	CORBA_exception_free (ev);
+
 	test_BasicServer_opException (objref, ev);
   
 	g_assert (ev->_major == CORBA_USER_EXCEPTION);
@@ -1037,8 +1042,9 @@ static void
 testMisc (test_TestFactory   factory, 
 	  CORBA_Environment *ev)
 {
-	CORBA_char   *foo;
-	CORBA_Context ctx;
+	CORBA_char       *foo;
+	CORBA_Context     ctx;
+	test_BasicServer  objref;
 
 	d_print ("Testing Misc bits...\n");
 
@@ -1050,29 +1056,30 @@ testMisc (test_TestFactory   factory,
 		CORBA_exception_free (ev);
 	}
 	
+
+	objref = test_TestFactory_getBasicServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (objref != CORBA_OBJECT_NIL);
+	g_assert (CORBA_Object_is_a (objref, "IDL:orbit/test/BasicServer:1.0", ev));
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	test_BasicServer_noImplement (objref, ev);
+	g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
+	g_assert (!strcmp (ev->_id, "IDL:CORBA/NO_IMPLEMENT:1.0"));
+	CORBA_exception_free (ev);
+
 	if (!in_proc) {
 		test_StrSeq *seq;
-		test_BasicServer objref;
-
-		objref = test_TestFactory_getBasicServer (factory, ev);
-		g_assert (ev->_major == CORBA_NO_EXCEPTION);
-		g_assert (objref != CORBA_OBJECT_NIL);
-		g_assert (CORBA_Object_is_a (objref, "IDL:orbit/test/BasicServer:1.0", ev));
-		g_assert (ev->_major == CORBA_NO_EXCEPTION);
-
-		test_BasicServer_noImplement (objref, ev);
-		g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
-		g_assert (!strcmp (ev->_id, "IDL:CORBA/NO_IMPLEMENT:1.0"));
-		CORBA_exception_free (ev);
 		
 		seq = make_large_str_seq ();
 		test_BasicServer_testLargeStringSeq (objref, seq, ev);
 		g_assert (ev->_major == CORBA_NO_EXCEPTION);
 		CORBA_free (seq);
 
-		CORBA_Object_release (objref, ev);
-		g_assert (ev->_major == CORBA_NO_EXCEPTION);
 	}
+
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 		
 	/* Check we are building full type data */
 	if (strcmp (TC_test_ObjectStruct->subtypes [0]->repo_id, 
@@ -1303,6 +1310,12 @@ testAsync (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
+static void
+broken_cb (LINCConnection *connection, gboolean *broken)
+{
+	*broken = TRUE;
+}
+
 static void
 testPingPong (test_TestFactory   factory, 
 	      CORBA_Environment *ev)
@@ -1358,7 +1371,55 @@ testPingPong (test_TestFactory   factory,
 	g_assert (!CORBA_Object_is_equivalent (
 		CORBA_OBJECT_NIL, l_objref, ev));
 
+	if (!in_proc) {
+		int i;
+		ORBitConnection *cnx = ORBit_small_get_connection (r_objref);
+		const char *base =
+			"This string is in order to provide some "
+			"more bulky data on the wire, the larger "
+			"the better. When the socket buffer fills "
+			"we start exercising the linc buffering code "
+			"and hopefully we get an exception somewhere "
+			"indicating that the buffer is full and that "
+			"the write has failed & that the connection is "
+			"now saturated ";
+		char *str;
+		gboolean broken;
+
+		g_assert (cnx != NULL);
+		ORBit_connection_set_max_buffer (cnx, 10000);
+
+		str = g_strdup (base);
+		for (i = 0; i < 4; i++) {
+			char *new;
+			new = g_strconcat (str, str, NULL);
+			g_free (str);
+			str = new;
+		}
+
+		d_print ("Testing non blocking IO ...\n");
+
+		ORBit_small_listen_for_broken (
+			r_objref, G_CALLBACK (broken_cb),
+			&broken);
+
+		for (i = 0; i < 100; i++) {
+			test_PingPongServer_opSleep (
+				r_objref, str, ev);
+			if (broken)
+				break;
+		}
+
+		g_free (str);
+
+		/* If this blows - perhaps you just have a strange
+		 * system scheduler */
+		g_assert (broken);
+		CORBA_exception_free (ev);
+	}
+
 	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
 	CORBA_Object_release (l_objref, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
@@ -1367,13 +1428,6 @@ testPingPong (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-static void
-broken_cb (LINCConnection *connection, gboolean *broken)
-{
-	*broken = TRUE;
-}
-
-
 static void
 dummy_cb (LINCConnection *connection, gboolean *invoked)
 {
