@@ -18,6 +18,8 @@
  * Author: Phil Dawes <philipd@users.sourceforge.net>
  */
 
+#undef DO_HARDER_SEGV
+
 #include "everything.h"
 #include <stdio.h>
 #include <string.h>
@@ -33,7 +35,8 @@ CORBA_Object create_object (PortableServer_POA poa,
 			    gpointer           servant,
 			    CORBA_Environment *ev);
 
-CORBA_ORB global_orb;
+CORBA_ORB          global_orb;
+PortableServer_POA global_poa;
 
 #include "basicServer.c"
 #include "structServer.c"
@@ -43,26 +46,22 @@ CORBA_ORB global_orb;
 #include "anyServer.c"
 #include "contextServer.c"
 #include "deadReference.c"
-
-/* Servant class */
+#include "pingServer.c"
 
 typedef struct {
 	POA_test_TestFactory baseServant;
 
-	test_BasicServer basicServerRef;
-	test_StructServer structServerRef;
-	test_SequenceServer sequenceServerRef;
-	test_UnionServer unionServerRef;
-	test_ArrayServer arrayServerRef;
-	test_AnyServer anyServerRef;
-	test_ContextServer contextServerRef;
+	test_BasicServer     basicServerRef;
+	test_StructServer    structServerRef;
+	test_SequenceServer  sequenceServerRef;
+	test_UnionServer     unionServerRef;
+	test_ArrayServer     arrayServerRef;
+	test_AnyServer       anyServerRef;
+	test_ContextServer   contextServerRef;
+	GSList              *pingPongServerRefs;
 } test_TestFactory_Servant;
 
-
-/* Methods */
-
-static
-test_BasicServer
+static test_BasicServer
 TestFactory_getBasicServer (PortableServer_Servant servant,
 			    CORBA_Environment     *ev)
 {
@@ -71,8 +70,7 @@ TestFactory_getBasicServer (PortableServer_Servant servant,
 	return CORBA_Object_duplicate (this->basicServerRef, ev);
 }
 
-static
-CORBA_char *
+static CORBA_char *
 TestFactory_getStructServerIOR (PortableServer_Servant servant,
 				CORBA_Environment     *ev)
 {
@@ -83,58 +81,81 @@ TestFactory_getStructServerIOR (PortableServer_Servant servant,
 		this->structServerRef, ev);
 }
 
-static
-test_StructServer
-TestFactory_getStructServer(PortableServer_Servant servant,
-			    CORBA_Environment *ev)
+static test_StructServer
+TestFactory_getStructServer (PortableServer_Servant servant,
+			     CORBA_Environment     *ev)
 {
-  test_TestFactory_Servant *this = (test_TestFactory_Servant*)servant;
-  return CORBA_Object_duplicate(this->structServerRef,ev);
+	test_TestFactory_Servant *this = (test_TestFactory_Servant*) servant;
+
+	return CORBA_Object_duplicate (this->structServerRef, ev);
 }
 
 static
 test_SequenceServer
 TestFactory_getSequenceServer(PortableServer_Servant servant,
-			      CORBA_Environment *ev)
+			      CORBA_Environment     *ev)
 {
-  test_TestFactory_Servant *this = (test_TestFactory_Servant*)servant;
-  return CORBA_Object_duplicate(this->sequenceServerRef,ev);
+	test_TestFactory_Servant *this = (test_TestFactory_Servant*) servant;
+
+	return CORBA_Object_duplicate(this->sequenceServerRef,ev);
 }
 
-static
-test_UnionServer
-TestFactory_getUnionServer(PortableServer_Servant servant,
-			   CORBA_Environment *ev)
+static test_UnionServer
+TestFactory_getUnionServer (PortableServer_Servant servant,
+			    CORBA_Environment     *ev)
 {
-  test_TestFactory_Servant *this = (test_TestFactory_Servant*)servant;
-  return CORBA_Object_duplicate(this->unionServerRef,ev);
+	test_TestFactory_Servant *this = (test_TestFactory_Servant*) servant;
+
+	return CORBA_Object_duplicate (this->unionServerRef, ev);
 }
 
-static
-test_ArrayServer
-TestFactory_getArrayServer(PortableServer_Servant servant,
-			   CORBA_Environment *ev)
+static test_ArrayServer
+TestFactory_getArrayServer (PortableServer_Servant servant,
+			    CORBA_Environment     *ev)
 {
-  test_TestFactory_Servant *this = (test_TestFactory_Servant*)servant;
-  return CORBA_Object_duplicate(this->arrayServerRef,ev);
+	test_TestFactory_Servant *this = (test_TestFactory_Servant*) servant;
+
+	return CORBA_Object_duplicate (this->arrayServerRef, ev);
 }
 
-static
-test_AnyServer
-TestFactory_getAnyServer(PortableServer_Servant servant,
-			 CORBA_Environment *ev)
+static test_AnyServer
+TestFactory_getAnyServer (PortableServer_Servant servant,
+			  CORBA_Environment     *ev)
 {
-  test_TestFactory_Servant *this = (test_TestFactory_Servant*)servant;
-  return CORBA_Object_duplicate(this->anyServerRef,ev);
+	test_TestFactory_Servant *this = (test_TestFactory_Servant*) servant;
+
+	return CORBA_Object_duplicate (this->anyServerRef, ev);
 }
 
-static
-test_ContextServer
-TestFactory_getContextServer(PortableServer_Servant servant,
-			     CORBA_Environment *ev)
+static test_ContextServer
+TestFactory_getContextServer (PortableServer_Servant servant,
+			      CORBA_Environment     *ev)
 {
-  test_TestFactory_Servant *this = (test_TestFactory_Servant*)servant;
-  return CORBA_Object_duplicate(this->contextServerRef,ev);
+	test_TestFactory_Servant *this = (test_TestFactory_Servant*) servant;
+
+	return CORBA_Object_duplicate (this->contextServerRef, ev);
+}
+
+static test_PingPongServer
+TestFactory_createPingPongServer (PortableServer_Servant servant,
+				  CORBA_Environment     *ev)
+{
+	test_PingPongServer obj;
+
+	obj = create_object (
+		global_poa, POA_test_PingPongServer__init,
+		create_ping_pong_servant (), ev);
+
+	if (servant) {
+		test_TestFactory_Servant *this;
+
+		this = (test_TestFactory_Servant*) servant;
+
+		this->pingPongServerRefs = g_slist_prepend (
+			this->pingPongServerRefs, obj);
+	}
+
+	return CORBA_Object_duplicate (obj, ev);
 }
 
 static test_DeadReferenceObj
@@ -159,9 +180,11 @@ TestFactory_createDeadReferenceObj (PortableServer_Servant  servant,
 	obj = create_object (poa, POA_test_DeadReferenceObj__init,
 			     &DeadReferenceObj_servant, ev);
 
-	/*
-	 * Note: Not duping - ORB will free it and reference should dangle.
-	 */
+	CORBA_Object_release ((CORBA_Object) poa, ev);
+	CORBA_Object_release ((CORBA_Object) poa_current, ev);
+
+	/* Note: Not duping - ORB will free it and reference
+	 * should dangle. */
 	return obj;
 }
 
@@ -170,8 +193,12 @@ TestFactory_segv (PortableServer_Servant servant,
 		  const CORBA_char      *when,
 		  CORBA_Environment     *ev)
 {
+#ifdef DO_HARDER_SEGV
 	/* Emulate a SegV */
 	exit (0);
+#else
+	g_main_loop_quit (linc_loop);
+#endif
 
 	return CORBA_string_dup ("I'm dead");
 }
@@ -180,15 +207,22 @@ static void
 test_TestFactory__fini (PortableServer_Servant  servant,
 			CORBA_Environment      *ev)
 {
-	test_TestFactory_Servant *this = (test_TestFactory_Servant*)servant;
+	GSList                   *l;
+	test_TestFactory_Servant *this;
 
-	CORBA_Object_release (this->basicServerRef,ev);
-	CORBA_Object_release (this->structServerRef,ev);
-	CORBA_Object_release (this->sequenceServerRef,ev);
-	CORBA_Object_release (this->unionServerRef,ev);
-	CORBA_Object_release (this->arrayServerRef,ev);
-	CORBA_Object_release (this->anyServerRef,ev);
-	CORBA_Object_release (this->contextServerRef,ev);
+	this = (test_TestFactory_Servant*) servant;
+
+	CORBA_Object_release (this->basicServerRef, ev);
+	CORBA_Object_release (this->structServerRef, ev);
+	CORBA_Object_release (this->sequenceServerRef, ev);
+	CORBA_Object_release (this->unionServerRef, ev);
+	CORBA_Object_release (this->arrayServerRef, ev);
+	CORBA_Object_release (this->anyServerRef, ev);
+	CORBA_Object_release (this->contextServerRef, ev);
+
+	for (l = this->pingPongServerRefs; l; l = l->next)
+		CORBA_Object_release (l->data, ev);
+	g_slist_free (this->pingPongServerRefs);
 }
 
 
@@ -216,6 +250,7 @@ static POA_test_TestFactory__epv TestFactory_epv = {
 	NULL,                         /* getDerivedServerAsB2         */
 	NULL,                         /* createTransientObj           */
 	TestFactory_createDeadReferenceObj,
+	TestFactory_createPingPongServer
 };
 
 static POA_test_TestFactory__vepv TestFactory_vepv = {
@@ -223,6 +258,22 @@ static POA_test_TestFactory__vepv TestFactory_vepv = {
 	&TestFactory_epv
 };
 
+static PortableServer_POA
+start_poa (CORBA_ORB orb, CORBA_Environment *ev)
+{
+	PortableServer_POAManager mgr;
+	PortableServer_POA the_poa;
+
+	the_poa = (PortableServer_POA) CORBA_ORB_resolve_initial_references (
+		orb, "RootPOA", ev);
+
+	mgr = PortableServer_POA__get_the_POAManager (the_poa, ev);
+	PortableServer_POAManager_activate (mgr, ev);
+	CORBA_Object_release ((CORBA_Object) mgr, ev);
+
+	return the_poa;
+}
+
 CORBA_Object
 create_object (PortableServer_POA poa,
 	       init_fn_t          fn,
@@ -257,7 +308,7 @@ create_object (PortableServer_POA poa,
 }
 
 /* constructor */
-void
+static void
 test_TestFactory__init (PortableServer_Servant servant, 
 			PortableServer_POA poa, 
 			CORBA_Environment *ev)
@@ -294,6 +345,8 @@ test_TestFactory__init (PortableServer_Servant servant,
 		poa, POA_test_ContextServer__init,
 		&ContextServer_servant, ev);
 
+	this->pingPongServerRefs = NULL;
+
 	POA_test_TestFactory__init (
 		(PortableServer_ServantBase *) servant, ev);
 }
@@ -331,22 +384,7 @@ dump_ior (CORBA_ORB orb, const char *fname, CORBA_Environment *ev)
 }
 #endif
 
-PortableServer_POA
-start_poa (CORBA_ORB orb, CORBA_Environment *ev)
-{
-	PortableServer_POAManager mgr;
-	PortableServer_POA the_poa;
-
-	the_poa = (PortableServer_POA) CORBA_ORB_resolve_initial_references (
-		orb, "RootPOA", ev);
-	mgr = PortableServer_POA__get_the_POAManager (the_poa, ev);
-	PortableServer_POAManager_activate (mgr, ev);
-	CORBA_Object_release ((CORBA_Object) mgr, ev);
-
-	return the_poa;
-}
-
-CORBA_Object
+static CORBA_Object
 create_TestFactory (PortableServer_POA        poa,
 		    test_TestFactory_Servant *servant,
 		    CORBA_Environment        *ev)
@@ -384,12 +422,11 @@ test_TestFactory_Servant servant;
   int
   main (int argc, char *argv [])
 #else
-  CORBA_Object
+  static CORBA_Object
   get_server (CORBA_ORB orb,
 	      CORBA_Environment *ev)
 #endif
 {
-	PortableServer_POA poa;
 	test_BasicServer objref;
 #ifndef _IN_CLIENT_
 	CORBA_Environment real_ev;
@@ -409,28 +446,33 @@ test_TestFactory_Servant servant;
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 #endif
 
-	poa = start_poa (global_orb, ev);
+	global_poa = start_poa (global_orb, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
-	factory = create_TestFactory (poa, &servant, ev);
-	CORBA_Object_release ((CORBA_Object)poa, ev);
+	factory = create_TestFactory (global_poa, &servant, ev);
 	g_assert (factory != CORBA_OBJECT_NIL);
 
 	/* a quick local test */
-	objref = test_TestFactory_getBasicServer(factory,ev);
-	g_assert(ev->_major == CORBA_NO_EXCEPTION);
-	g_assert(objref != CORBA_OBJECT_NIL);
-	g_assert(CORBA_Object_is_a (objref, "IDL:orbit/test/BasicServer:1.0", ev));
-	g_assert(ev->_major == CORBA_NO_EXCEPTION);
+	objref = test_TestFactory_getBasicServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (objref != CORBA_OBJECT_NIL);
+	g_assert (CORBA_Object_is_a (objref, "IDL:orbit/test/BasicServer:1.0", ev));
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 	CORBA_Object_release (objref, ev);
 	g_assert(ev->_major == CORBA_NO_EXCEPTION);
 	fprintf (stderr, "Local server test passed\n");
 
 #ifndef _IN_CLIENT_
-	if (!dump_ior (global_orb, "iorfile", ev)) {
+	if (!dump_ior (global_orb, "iorfile", ev))
 		CORBA_ORB_run (global_orb, ev);
-		return 0;
-	}
+
+	CORBA_Object_release ((CORBA_Object) global_poa, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	CORBA_Object_release (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	g_warning ("released factory");
 
 	CORBA_ORB_destroy (global_orb, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
@@ -439,7 +481,8 @@ test_TestFactory_Servant servant;
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 	
 	CORBA_exception_free (ev);
-	return 1;
+
+	return 0;
 #else
 	return factory;
 #endif
